@@ -31,7 +31,7 @@
 #if ENABLE(GRAPHICS_CONTEXT_GL) && USE(TEXTURE_MAPPER)
 
 #include "GraphicsContextGLOpenGLPrivate.h"
-#include "TextureMapperGC3DPlatformLayer.h"
+#include "TextureMapperGCGLPlatformLayer.h"
 #include <ANGLE/ShaderLang.h>
 #include <wtf/Deque.h>
 #include <wtf/NeverDestroyed.h>
@@ -75,9 +75,9 @@ typedef void* GLeglContext;
 
 #if USE(NICOSIA)
 #if USE(ANGLE)
-#include "NicosiaGC3DANGLELayer.h"
+#include "NicosiaGCGLANGLELayer.h"
 #else
-#include "NicosiaGC3DLayer.h"
+#include "NicosiaGCGLLayer.h"
 #endif
 #endif
 
@@ -135,9 +135,9 @@ GraphicsContextGLOpenGL::GraphicsContextGLOpenGL(GraphicsContextGLAttributes att
 {
     ASSERT_UNUSED(sharedContext, !sharedContext);
 #if USE(NICOSIA)
-    m_nicosiaLayer = WTF::makeUnique<Nicosia::GC3DANGLELayer>(*this, destination);
+    m_nicosiaLayer = WTF::makeUnique<Nicosia::GCGLANGLELayer>(*this, destination);
 #else
-    m_texmapLayer = WTF::makeUnique<TextureMapperGC3DPlatformLayer>(*this, destination);
+    m_texmapLayer = WTF::makeUnique<TextureMapperGCGLPlatformLayer>(*this, destination);
 #endif
     makeContextCurrent();
 
@@ -177,17 +177,18 @@ GraphicsContextGLOpenGL::GraphicsContextGLOpenGL(GraphicsContextGLAttributes att
 #endif
 
         // Create a multisample FBO.
+        ASSERT(m_state.boundReadFBO == m_state.boundDrawFBO);
         if (attributes.antialias) {
             gl::GenFramebuffers(1, &m_multisampleFBO);
             gl::BindFramebuffer(GL_FRAMEBUFFER, m_multisampleFBO);
-            m_state.boundFBO = m_multisampleFBO;
+            m_state.boundDrawFBO = m_state.boundReadFBO = m_multisampleFBO;
             gl::GenRenderbuffers(1, &m_multisampleColorBuffer);
             if (attributes.stencil || attributes.depth)
                 gl::GenRenderbuffers(1, &m_multisampleDepthStencilBuffer);
         } else {
             // Bind canvas FBO.
             gl::BindFramebuffer(GL_FRAMEBUFFER, m_fbo);
-            m_state.boundFBO = m_fbo;
+            m_state.boundDrawFBO = m_state.boundReadFBO = m_fbo;
 #if USE(OPENGL_ES)
             if (attributes.depth)
                 gl::GenRenderbuffers(1, &m_depthBuffer);
@@ -207,9 +208,9 @@ GraphicsContextGLOpenGL::GraphicsContextGLOpenGL(GraphicsContextGLAttributes att
 {
     ASSERT_UNUSED(sharedContext, !sharedContext);
 #if USE(NICOSIA)
-    m_nicosiaLayer = makeUnique<Nicosia::GC3DLayer>(*this, destination);
+    m_nicosiaLayer = makeUnique<Nicosia::GCGLLayer>(*this, destination);
 #else
-    m_texmapLayer = makeUnique<TextureMapperGC3DPlatformLayer>(*this, destination);
+    m_texmapLayer = makeUnique<TextureMapperGCGLPlatformLayer>(*this, destination);
 #endif
 
     makeContextCurrent();
@@ -250,17 +251,18 @@ GraphicsContextGLOpenGL::GraphicsContextGLOpenGL(GraphicsContextGLAttributes att
 #endif
 
         // Create a multisample FBO.
+        ASSERT(m_state.boundReadFBO == m_state.boundDrawFBO);
         if (attributes.antialias) {
             ::glGenFramebuffers(1, &m_multisampleFBO);
             ::glBindFramebuffer(GL_FRAMEBUFFER, m_multisampleFBO);
-            m_state.boundFBO = m_multisampleFBO;
+            m_state.boundDrawFBO = m_state.boundReadFBO = m_multisampleFBO;
             ::glGenRenderbuffers(1, &m_multisampleColorBuffer);
             if (attributes.stencil || attributes.depth)
                 ::glGenRenderbuffers(1, &m_multisampleDepthStencilBuffer);
         } else {
             // Bind canvas FBO.
             glBindFramebuffer(GraphicsContextGLOpenGL::FRAMEBUFFER, m_fbo);
-            m_state.boundFBO = m_fbo;
+            m_state.boundDrawFBO = m_state.boundReadFBO = m_fbo;
 #if USE(OPENGL_ES)
             if (attributes.depth)
                 glGenRenderbuffers(1, &m_depthBuffer);
