@@ -46,6 +46,8 @@
 #include <WebCore/PathUtilities.h>
 #include <WebCore/PlatformMouseEvent.h>
 #include <WebCore/PluginDocument.h>
+#include <WebCore/Range.h>
+#include <WebCore/SimpleRange.h>
 
 #if PLATFORM(COCOA)
 #include <WebCore/TextIndicatorWindow.h>
@@ -54,18 +56,18 @@
 namespace WebKit {
 using namespace WebCore;
 
-WebCore::FindOptions core(FindOptions options)
+WebCore::FindOptions core(OptionSet<FindOptions> options)
 {
     WebCore::FindOptions result;
-    if (options & FindOptionsCaseInsensitive)
+    if (options.contains(FindOptions::CaseInsensitive))
         result.add(WebCore::CaseInsensitive);
-    if (options & FindOptionsAtWordStarts)
+    if (options.contains(FindOptions::AtWordStarts))
         result.add(WebCore::AtWordStarts);
-    if (options & FindOptionsTreatMedialCapitalAsWordStart)
+    if (options.contains(FindOptions::TreatMedialCapitalAsWordStart))
         result.add(WebCore::TreatMedialCapitalAsWordStart);
-    if (options & FindOptionsBackwards)
+    if (options.contains(FindOptions::Backwards))
         result.add(WebCore::Backwards);
-    if (options & FindOptionsWrapAround)
+    if (options.contains(FindOptions::WrapAround))
         result.add(WebCore::WrapAround);
     return result;
 }
@@ -79,7 +81,7 @@ FindController::~FindController()
 {
 }
 
-void FindController::countStringMatches(const String& string, FindOptions options, unsigned maxMatchCount)
+void FindController::countStringMatches(const String& string, OptionSet<FindOptions> options, unsigned maxMatchCount)
 {
     if (maxMatchCount == std::numeric_limits<unsigned>::max())
         --maxMatchCount;
@@ -110,11 +112,11 @@ uint32_t FindController::replaceMatches(const Vector<uint32_t>& matchIndices, co
     // periodically update an NSProgress in the UI process when a batch of find-in-page matches are replaced.
     const uint32_t maximumNumberOfMatchesToReplace = 1000;
 
-    Vector<Ref<Range>> rangesToReplace;
+    Vector<SimpleRange> rangesToReplace;
     rangesToReplace.reserveCapacity(std::min<uint32_t>(maximumNumberOfMatchesToReplace, matchIndices.size()));
     for (auto index : matchIndices) {
         if (index < m_findMatches.size())
-            rangesToReplace.uncheckedAppend(*m_findMatches[index]);
+            rangesToReplace.uncheckedAppend(m_findMatches[index]);
         if (rangesToReplace.size() >= maximumNumberOfMatchesToReplace)
             break;
     }
@@ -131,7 +133,7 @@ static Frame* frameWithSelection(Page* page)
     return 0;
 }
 
-void FindController::updateFindUIAfterPageScroll(bool found, const String& string, FindOptions options, unsigned maxMatchCount, DidWrap didWrap, FindUIOriginator originator)
+void FindController::updateFindUIAfterPageScroll(bool found, const String& string, OptionSet<FindOptions> options, unsigned maxMatchCount, DidWrap didWrap, FindUIOriginator originator)
 {
     Frame* selectedFrame = frameWithSelection(m_webPage->corePage());
     
@@ -152,9 +154,9 @@ void FindController::updateFindUIAfterPageScroll(bool found, const String& strin
 
         m_webPage->send(Messages::WebPageProxy::DidFailToFindString(string));
     } else {
-        shouldShowOverlay = options & FindOptionsShowOverlay;
-        bool shouldShowHighlight = options & FindOptionsShowHighlight;
-        bool shouldDetermineMatchIndex = options & FindOptionsDetermineMatchIndex;
+        shouldShowOverlay = options.contains(FindOptions::ShowOverlay);
+        bool shouldShowHighlight = options.contains(FindOptions::ShowHighlight);
+        bool shouldDetermineMatchIndex = options.contains(FindOptions::DetermineMatchIndex);
         unsigned matchCount = 1;
 
         if (shouldDetermineMatchIndex) {
@@ -201,10 +203,9 @@ void FindController::updateFindUIAfterPageScroll(bool found, const String& strin
             m_findMatches.clear();
             Vector<IntRect> matchRects;
             if (auto range = m_webPage->corePage()->selection().firstRange()) {
-                range->absoluteTextRects(matchRects);
-                m_findMatches.append(range);
+                matchRects = RenderObject::absoluteTextRects(*range);
+                m_findMatches.append(*range);
             }
-
             m_webPage->send(Messages::WebPageProxy::DidFindString(string, matchRects, matchCount, m_foundStringMatchIndex, didWrap == DidWrap::Yes));
         }
     }
@@ -221,11 +222,11 @@ void FindController::updateFindUIAfterPageScroll(bool found, const String& strin
         m_findPageOverlay->setNeedsDisplay();
     }
     
-    if (found && (!(options & FindOptionsShowFindIndicator) || !selectedFrame || !updateFindIndicator(*selectedFrame, shouldShowOverlay)))
+    if (found && (!options.contains(FindOptions::ShowFindIndicator) || !selectedFrame || !updateFindIndicator(*selectedFrame, shouldShowOverlay)))
         hideFindIndicator();
 }
 
-void FindController::findString(const String& string, FindOptions options, unsigned maxMatchCount, Optional<CallbackID> callbackID)
+void FindController::findString(const String& string, OptionSet<FindOptions> options, unsigned maxMatchCount, CompletionHandler<void(bool)>&& completionHandler)
 {
     auto* pluginView = WebPage::pluginViewForFrame(m_webPage->mainFrame());
 
@@ -243,12 +244,10 @@ void FindController::findString(const String& string, FindOptions options, unsig
     bool foundStringStartsAfterSelection = false;
     if (!pluginView) {
         if (Frame* selectedFrame = frameWithSelection(m_webPage->corePage())) {
-            FrameSelection& fs = selectedFrame->selection();
-            if (fs.selectionBounds().isEmpty()) {
-                m_findMatches.clear();
-                int indexForSelection;
-                m_webPage->corePage()->findStringMatchingRanges(string, coreOptions, maxMatchCount, m_findMatches, indexForSelection);
-                m_foundStringMatchIndex = indexForSelection;
+            if (selectedFrame->selection().selectionBounds().isEmpty()) {
+                auto result = m_webPage->corePage()->findTextMatches(string, coreOptions, maxMatchCount);
+                m_findMatches = WTFMove(result.ranges);
+                m_foundStringMatchIndex = result.indexForSelection;
                 foundStringStartsAfterSelection = true;
             }
         }
@@ -267,9 +266,9 @@ void FindController::findString(const String& string, FindOptions options, unsig
         didFindString();
 
         if (!foundStringStartsAfterSelection) {
-            if (options & FindOptionsBackwards)
+            if (options.contains(FindOptions::Backwards))
                 m_foundStringMatchIndex--;
-            else if (!(options & FindOptionsNoIndexChange))
+            else if (!options.contains(FindOptions::NoIndexChange))
                 m_foundStringMatchIndex++;
         }
     }
@@ -279,27 +278,21 @@ void FindController::findString(const String& string, FindOptions options, unsig
         protectedWebPage->findController().updateFindUIAfterPageScroll(found, string, options, maxMatchCount, didWrap, FindUIOriginator::FindString);
     });
 
-    if (callbackID)
-        m_webPage->send(Messages::WebPageProxy::FindStringCallback(found, *callbackID));
+    completionHandler(found);
 }
 
-void FindController::findStringMatches(const String& string, FindOptions options, unsigned maxMatchCount)
+void FindController::findStringMatches(const String& string, OptionSet<FindOptions> options, unsigned maxMatchCount)
 {
-    m_findMatches.clear();
-    int indexForSelection;
-
-    m_webPage->corePage()->findStringMatchingRanges(string, core(options), maxMatchCount, m_findMatches, indexForSelection);
+    auto result = m_webPage->corePage()->findTextMatches(string, core(options), maxMatchCount);
+    m_findMatches = WTFMove(result.ranges);
 
     Vector<Vector<IntRect>> matchRects;
-    for (size_t i = 0; i < m_findMatches.size(); ++i) {
-        Vector<IntRect> rects;
-        m_findMatches[i]->absoluteTextRects(rects);
-        matchRects.append(WTFMove(rects));
-    }
+    for (auto& range : m_findMatches)
+        matchRects.append(RenderObject::absoluteTextRects(range));
 
-    m_webPage->send(Messages::WebPageProxy::DidFindStringMatches(string, matchRects, indexForSelection));
+    m_webPage->send(Messages::WebPageProxy::DidFindStringMatches(string, matchRects, result.indexForSelection));
 
-    if (!(options & FindOptionsShowOverlay || options & FindOptionsShowFindIndicator))
+    if (!options.contains(FindOptions::ShowOverlay) && !options.contains(FindOptions::ShowFindIndicator))
         return;
 
     bool found = !m_findMatches.isEmpty();
@@ -312,12 +305,12 @@ void FindController::getImageForFindMatch(uint32_t matchIndex)
 {
     if (matchIndex >= m_findMatches.size())
         return;
-    Frame* frame = m_findMatches[matchIndex]->startContainer().document().frame();
+    Frame* frame = m_findMatches[matchIndex].start.container->document().frame();
     if (!frame)
         return;
 
     VisibleSelection oldSelection = frame->selection().selection();
-    frame->selection().setSelection(VisibleSelection(*m_findMatches[matchIndex]));
+    frame->selection().setSelection(m_findMatches[matchIndex]);
 
     RefPtr<ShareableBitmap> selectionSnapshot = WebFrame::fromCoreFrame(*frame)->createSelectionSnapshot();
 
@@ -343,10 +336,10 @@ void FindController::selectFindMatch(uint32_t matchIndex)
 {
     if (matchIndex >= m_findMatches.size())
         return;
-    Frame* frame = m_findMatches[matchIndex]->startContainer().document().frame();
+    Frame* frame = m_findMatches[matchIndex].start.container->document().frame();
     if (!frame)
         return;
-    frame->selection().setSelection(VisibleSelection(*m_findMatches[matchIndex]));
+    frame->selection().setSelection(m_findMatches[matchIndex]);
 }
 
 void FindController::indicateFindMatch(uint32_t matchIndex)
@@ -381,7 +374,7 @@ void FindController::hideFindUI()
 
 bool FindController::updateFindIndicator(Frame& selectedFrame, bool isShowingOverlay, bool shouldAnimate)
 {
-    auto indicator = TextIndicator::createWithSelectionInFrame(selectedFrame, TextIndicatorOptionIncludeMarginIfRangeMatchesSelection, shouldAnimate ? TextIndicatorPresentationTransition::Bounce : TextIndicatorPresentationTransition::None);
+    auto indicator = TextIndicator::createWithSelectionInFrame(selectedFrame, { TextIndicatorOption::IncludeMarginIfRangeMatchesSelection }, shouldAnimate ? TextIndicatorPresentationTransition::Bounce : TextIndicatorPresentationTransition::None);
     if (!indicator)
         return false;
 
@@ -512,8 +505,8 @@ void FindController::drawRect(PageOverlay&, GraphicsContext& graphicsContext, co
 {
     const int borderWidth = 1;
 
-    Color overlayBackgroundColor(0.1f, 0.1f, 0.1f, 0.25f);
-    Color shadowColor(0.0f, 0.0f, 0.0f, 0.5f);
+    constexpr auto overlayBackgroundColor = SRGBA<uint8_t> { 26, 26, 26, 64 };
+    constexpr auto shadowColor = Color::black.colorWithAlphaByte(128);
 
     IntRect borderInflatedDirtyRect = dirtyRect;
     borderInflatedDirtyRect.inflate(borderWidth);
